#!/usr/bin/env python3

from os import environ, path
from github import Github
from lib.release import create_release

WORK_DIR = path.dirname(path.abspath(__file__)).replace('/tools/release', '')

NEXT_TAG = environ.get('NEXT_RELEASE_TAG', None)
REPO_NAME = environ.get('GITHUB_REPOSITORY', None)
TOKEN = environ.get('GITHUB_TOKEN', None)
if not NEXT_TAG or not REPO_NAME or not TOKEN:
    raise Exception('Bad environment variables. Invalid GITHUB_REPOSITORY, GITHUB_TOKEN or NEXT_RELEASE_TAG')

g = Github(TOKEN)
repository = g.get_repo(REPO_NAME)

release = create_release(repository, NEXT_TAG)
release.upload_asset('singleheader/ada.cpp')
release.upload_asset('singleheader/ada.h')
release.upload_asset('singleheader/ada_c.h')
release.upload_asset('singleheader/singleheader.zip')
release.update_release(name=release.name, message=release.body, draft=False, make_latest="true")



#!/usr/bin/env python3
#
# Creates the amalgamated source files.
#

import zipfile
import sys
import os.path
import subprocess
import os
import re
import shutil
import datetime
from typing import TextIO

if sys.version_info[0] < 3:
    sys.stdout.write('Sorry, requires Python 3.x or better\n')
    sys.exit(1)

SCRIPT_PATH = os.path.dirname(os.path.abspath(sys.argv[0]))
PROJECT_PATH = os.path.dirname(SCRIPT_PATH)
print(f'SCRIPT_PATH={SCRIPT_PATH} PROJECT_PATH={PROJECT_PATH}')

AMALGAMATE_SOURCE_PATH = os.environ.get('AMALGAMATE_SOURCE_PATH') or os.path.join(PROJECT_PATH, 'src')
AMALGAMATE_INCLUDE_PATH = os.environ.get('AMALGAMATE_INCLUDE_PATH') or os.path.join(PROJECT_PATH, 'include')
AMALGAMATE_OUTPUT_PATH = os.environ.get('AMALGAMATE_OUTPUT_PATH') or os.path.join(SCRIPT_PATH)

# this list excludes the "src/generic headers"
ALL_C_FILES = ['ada.cpp']

# order matters
ALL_C_HEADERS = ['ada.h']

found_includes = []

def doinclude(fid: TextIO, file: str, line: str, origin: str) -> None:
    p = os.path.join(AMALGAMATE_INCLUDE_PATH, file)
    pi = os.path.join(AMALGAMATE_SOURCE_PATH, file)

    if os.path.exists(p):
        if file not in found_includes:
            found_includes.append(file)
            dofile(fid, AMALGAMATE_INCLUDE_PATH, file)
    elif os.path.exists(pi):
        if file not in found_includes:
            found_includes.append(file)
            dofile(fid, AMALGAMATE_SOURCE_PATH, file)
    else:
        # If we don't recognize it, just emit the #include
        print('unrecognized:', file, ' from ', line, ' in ', origin)
        print(line, file=fid)


def dofile(fid: TextIO, prepath: str, filename: str) -> None:
    file = os.path.join(prepath, filename)
    RELFILE = os.path.relpath(file, PROJECT_PATH)
    # Last lines are always ignored. Files should end by an empty lines.
    print(f'/* begin file {RELFILE} */', file=fid)
    includepattern = re.compile('\\s*#\\s*include "(.*)"')
    with open(file, 'r') as fid2:
        for line in fid2:
            line = line.rstrip('\n')
            s = includepattern.search(line)
            if s:
                includedfile = s.group(1)
                if includedfile == 'ada.h' and filename == 'ada.cpp':
                    print(line, file=fid)
                    continue

                if includedfile.startswith('../'):
                    includedfile = includedfile[2:]
                # we explicitly include ada headers, one time each
                doinclude(fid, includedfile, line, filename)
            else:
                print(line, file=fid)
    print(f'/* end file {RELFILE} */', file=fid)


# Get the generation date from git, so the output is reproducible.
# The %ci specifier gives the unambiguous ISO 8601 format, and
# does not change with locale and timezone at time of generation.
# Forcing it to be UTC is difficult, because it needs to be portable
# between gnu date and busybox date.
try:
    timestamp = (
        subprocess.run(['git', 'show', '-s', '--format=%ci', 'HEAD'], stdout=subprocess.PIPE)
        .stdout.decode('utf-8')
        .strip()
    )
except Exception:
    print('git not found, timestamp based on current time')
    timestamp = str(datetime.datetime.now())
print(f'timestamp is {timestamp}')

os.makedirs(AMALGAMATE_OUTPUT_PATH, exist_ok=True)
AMAL_H = os.path.join(AMALGAMATE_OUTPUT_PATH, 'ada.h')
AMAL_C = os.path.join(AMALGAMATE_OUTPUT_PATH, 'ada.cpp')
DEMOCPP = os.path.join(AMALGAMATE_OUTPUT_PATH, 'cpp')
README = os.path.join(AMALGAMATE_OUTPUT_PATH, 'README.md')

print(f'Creating {AMAL_H}')
amal_h = open(AMAL_H, mode='w', encoding='utf8')
print(f'/* auto-generated on {timestamp}. Do not edit! */', file=amal_h)
for h in ALL_C_HEADERS:
    doinclude(amal_h, h, f'ERROR {h} not found', h)

amal_h.close()
print(f'Creating {AMAL_C}')
amal_c = open(AMAL_C, mode='w', encoding='utf8')
print(f'/* auto-generated on {timestamp}. Do not edit! */', file=amal_c)
for c in ALL_C_FILES:
    doinclude(amal_c, c, f'ERROR {c} not found', c)

amal_c.close()

# copy the README and DEMOCPP
if SCRIPT_PATH != AMALGAMATE_OUTPUT_PATH:
    shutil.copy2(os.path.join(SCRIPT_PATH, 'demo.cpp'), AMALGAMATE_OUTPUT_PATH)
    shutil.copy2(os.path.join(SCRIPT_PATH, 'demo.c'), AMALGAMATE_OUTPUT_PATH)
    shutil.copy2(os.path.join(SCRIPT_PATH, 'README.md'), AMALGAMATE_OUTPUT_PATH)

shutil.copy2(os.path.join(AMALGAMATE_INCLUDE_PATH, 'ada_c.h'), AMALGAMATE_OUTPUT_PATH)

zf = zipfile.ZipFile(os.path.join(AMALGAMATE_OUTPUT_PATH, 'singleheader.zip'), 'w', zipfile.ZIP_DEFLATED)
zf.write(os.path.join(AMALGAMATE_OUTPUT_PATH, 'ada.cpp'), 'ada.cpp')
zf.write(os.path.join(AMALGAMATE_OUTPUT_PATH, 'ada.h'), 'ada.h')
zf.write(os.path.join(AMALGAMATE_INCLUDE_PATH, 'ada_c.h'), 'ada_c.h')


print('Done with all files generation.')

print(f'Files have been written to directory: {AMALGAMATE_OUTPUT_PATH}/')
print('Done with all files generation.')



#!/usr/bin/env python3

import fileinput
import re


def update_cmakelists_version(new_version: str, file_path: str) -> None:
    inside_project = False
    with fileinput.FileInput(file_path, inplace=True) as cmakelists:
        for line in cmakelists:
            if 'set(ADA_LIB_VERSION' in line:
                line = re.sub(r'[0-9]+\.[0-9]+\.[0-9]+', new_version, line)
            elif 'set(ADA_LIB_SOVERSION' in line:
                line = re.sub(r'[0-9]+', new_version.split('.')[0], line)

            elif 'project(' in line:
                inside_project = True
            elif inside_project:
                if 'VERSION' in line:
                    line = re.sub(r'[0-9]+\.[0-9]+\.[0-9]+', new_version, line)
                    inside_project = False
            print(line, end='')


def update_ada_version_h(new_version: str, file_path: str) -> None:
    new_version_list = new_version.split('.')
    with fileinput.FileInput(file_path, inplace=True) as ada_version_h:
        inside_enum = False
        for line in ada_version_h:
            if '#define ADA_VERSION' in line:
                line = f'#define ADA_VERSION "{new_version}"\n'

            elif 'enum {' in line:
                inside_enum = True
            elif inside_enum:
                if line.strip().startswith('ADA_VERSION_MAJOR'):
                    line = re.sub(r'\d+', new_version_list[0], line)
                elif line.strip().startswith('ADA_VERSION_MINOR'):
                    line = re.sub(r'\d+', new_version_list[1], line)
                elif line.strip().startswith('ADA_VERSION_REVISION'):
                    line = re.sub(r'\d+', new_version_list[2], line)

            print(line, end='')


def update_doxygen_version(new_version: str, file_path: str) -> None:
    with fileinput.FileInput(file_path, inplace=True) as doxygen:
        for line in doxygen:
            if line.strip().startswith('PROJECT_NUMBER         ='):
                line = f'PROJECT_NUMBER         = "{new_version}"\n'

            print(line, end='')



from .. import release
from datetime import datetime
from typing import NamedTuple


class Release(NamedTuple):
    title: str
    created_at: datetime


class User(NamedTuple):
    login: str


class CommitMessage(NamedTuple):
    message: str


class Commit(NamedTuple):
    author: User
    commit: CommitMessage


class PullRequestBase(NamedTuple):
    ref: str


class PullRequest(NamedTuple):
    title: str
    number: int
    state: str
    base: PullRequestBase
    merged: bool
    merged_at: datetime
    user: User
    commits: list[Commit]

    def get_commits(self) -> list[Commit]:
        return self.commits


class RepoStub:
    def __init__(self) -> None:
        self.created_at = datetime(2023, 1, 1)
        self.full_name = 'ada-url/ada'

    @staticmethod
    def get_releases() -> list[Release]:
        return [
            Release('v1.0.1', datetime(2023, 2, 1)),
            Release('v1.0.3', datetime(2023, 4, 1)),
            Release('v1.0.2', datetime(2023, 3, 1)),
        ]

    @staticmethod
    def get_pulls(state: str = 'closed') -> list[PullRequest]:
        return list(
            filter(
                lambda pull: pull.state == state,
                [
                    PullRequest(
                        title='Feature 1',
                        number=1,
                        state='open',
                        merged=False,
                        base=PullRequestBase('main'),
                        merged_at=datetime(2023, 2, 2),
                        user=User('contributor_1'),
                        commits=[
                            Commit(
                                User('contributor_1'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_1'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_1'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_1'),
                                CommitMessage('src: sample commit 4'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 2',
                        number=2,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 2, 1),
                        base=PullRequestBase('main'),
                        user=User('contributor_2'),
                        commits=[
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('Co-authored-by: old_contrib_coauthor2 <the@email>'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 4'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('Co-authored-by: old_contrib_coauthor <the@email>'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 3',
                        number=3,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 2, 2),
                        base=PullRequestBase('main'),
                        user=User('contributor_3'),
                        commits=[
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 4'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 4',
                        number=4,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 2, 3),
                        base=PullRequestBase('main'),
                        user=User('contributor_4'),
                        commits=[
                            Commit(
                                User('contributor_4'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_4'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_4'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_4'),
                                CommitMessage('src: sample commit 4'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 5',
                        number=5,
                        state='closed',
                        merged=False,
                        merged_at=datetime(2023, 2, 4),
                        base=PullRequestBase('main'),
                        user=User('contributor_3'),
                        commits=[
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 4'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 6',
                        number=12,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 2, 5),
                        base=PullRequestBase('main'),
                        user=User('contributor_2'),
                        commits=[
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_2'),
                                CommitMessage('src: sample commit 4'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 9',
                        number=13,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 5, 2),
                        base=PullRequestBase('main'),
                        user=User('new_contributor_2'),
                        commits=[
                            Commit(
                                User('new_contributor_2'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('new_contributor_2'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('new_contributor_2'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('new_contributor_2'),
                                CommitMessage('src: sample commit 4 '),
                            ),
                            Commit(
                                User('new_contributor_2'),
                                CommitMessage('Co-authored-by: new_contributor_coauthor1 <the@email>'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 7',
                        number=14,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 5, 5),
                        base=PullRequestBase('main'),
                        user=User('contributor_3'),
                        commits=[
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('src: sample commit 4 '),
                            ),
                            Commit(
                                User('contributor_3'),
                                CommitMessage('Co-authored-by: new_contributor_coauthor2 <the@email>'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 8',
                        number=15,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 5, 1),
                        base=PullRequestBase('main'),
                        user=User('new_contributor_1'),
                        commits=[
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 2'),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 3'),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('Co-authored-by: new_contributor_coauthor4 <the@email>'),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 4 '),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('Co-authored-by: new_contributor_coauthor3 <the@email>'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='Feature 11',
                        number=16,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 5, 10),
                        base=PullRequestBase('another_branch'),
                        user=User('new_contributor_1'),
                        commits=[
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 2'),
                            ),
                        ],
                    ),
                    PullRequest(
                        title='chore: release v10.0.0',
                        number=17,
                        state='closed',
                        merged=True,
                        merged_at=datetime(2023, 5, 9),
                        base=PullRequestBase('main'),
                        user=User('new_contributor_1'),
                        commits=[
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 1'),
                            ),
                            Commit(
                                User('new_contributor_1'),
                                CommitMessage('src: sample commit 2'),
                            ),
                        ],
                    ),
                ],
            )
        )


def test_is_valid_tag() -> None:
    assert release.is_valid_tag('v1.0.0') is True
    assert release.is_valid_tag('v1.1.1') is True

    assert release.is_valid_tag('v0') is False
    assert release.is_valid_tag('v1.0.0.0') is False
    assert release.is_valid_tag('1.0.0') is False
    assert release.is_valid_tag('1.0.1') is False



#!/usr/bin/env python3

import re
from github.Repository import Repository
from github.GitRelease import GitRelease


def is_valid_tag(tag: str) -> bool:
    tag_regex = r'^v\d+\.\d+\.\d+$'
    return bool(re.match(tag_regex, tag))


def create_release(repository: Repository, tag: str) -> GitRelease:
    if not is_valid_tag(tag):
        raise Exception(f'Invalid tag: {tag}')

    try:
        return repository.create_git_release(
            tag=tag, name=tag, draft=True, prerelease=False, generate_release_notes=True
        )
    except Exception as exp:
        raise Exception(f'create_release: Error creating release/tag {tag}: {exp!s}') from exp



#!/usr/bin/env python3

import os
import lib.versions as update_versions
from lib.release import is_valid_tag

WORK_DIR = os.path.dirname(os.path.abspath(__file__)).replace('/tools/release', '')

ADA_VERSION_H = f'{WORK_DIR}/include/ada/ada_version.h'
DOXYGEN = f'{WORK_DIR}/doxygen'
CMAKE_LISTS = f'{WORK_DIR}/CMakeLists.txt'

NEXT_TAG = os.environ['NEXT_RELEASE_TAG']
if not NEXT_TAG or not is_valid_tag(NEXT_TAG):
    raise Exception(f'Bad environment variables. Invalid NEXT_RELEASE_TAG {NEXT_TAG}.')

NEXT_TAG = NEXT_TAG[1:]  # from v1.0.0 to 1.0.0

update_versions.update_ada_version_h(NEXT_TAG, ADA_VERSION_H)
update_versions.update_doxygen_version(NEXT_TAG, DOXYGEN)
update_versions.update_cmakelists_version(NEXT_TAG, CMAKE_LISTS)
